package Math::BigInt::FastCalc;

use 5.006;
use strict;

use DynaLoader;
use Math::BigInt::Calc;

use vars qw/@ISA $VERSION $BASE $BASE_LEN/;

@ISA = qw(DynaLoader);

$VERSION = '0.16';

bootstrap Math::BigInt::FastCalc $VERSION;


sub api_version () { 1; }
 
BEGIN
  {
  # use Calc to override the methods that we do not provide in XS

  for my $method (qw/
    str
    add sub mul div
    rsft lsft
    mod modpow modinv
    gcd
    pow root sqrt log_int fac nok
    digit check
    from_hex from_bin from_oct as_hex as_bin as_oct
    zeros base_len
    xor or and
    alen 1ex
    /)
    {
    no strict 'refs';
    *{'Math::BigInt::FastCalc::_' . $method} = \&{'Math::BigInt::Calc::_' . $method};
    }
  my ($AND_BITS, $XOR_BITS, $OR_BITS, $BASE_LEN_SMALL, $MAX_VAL);
 
  # store BASE_LEN and BASE to later pass it to XS code 
  ($BASE_LEN, $AND_BITS, $XOR_BITS, $OR_BITS, $BASE_LEN_SMALL, $MAX_VAL, $BASE) =
    Math::BigInt::Calc::_base_len();

  }

sub import
  {
  _set_XS_BASE($BASE, $BASE_LEN);
  }


1;
__END__

